@@ -291,7 +291,7 @@ def pswapaxes(x, axis_name, axis, *, axis_index_groups=None):
   """
   return all_to_all(x, axis_name, axis, axis, axis_index_groups=axis_index_groups)
 
-def all_to_all(x, axis_name, split_axis, concat_axis, *, axis_index_groups=None):
+def all_to_all(x, axis_name, split_axis, concat_axis, *, axis_index_groups=None, tiled=False):
   """Materialize the mapped axis and map a different axis.
 
   If ``x`` is a pytree then the result is equivalent to mapping this function to
@@ -318,24 +318,47 @@ def all_to_all(x, axis_name, split_axis, concat_axis, *, axis_index_groups=None)
       an axis of size 4, [[0, 1], [2, 3]] would run all_to_all over the first
       two and last two replicas). Groups must cover all axis indices exactly
       once, and all groups must be the same size.
+    tiled: when True, all_to_all will divide split_axis into chunks and concatenate
+      them along concat_axis. In particular, no dimensions are added or removed.
+      False by default.
 
   Returns:
-    Array(s) with shape given by the expression::
+    When tiled is False, array(s) with shape given by the expression::
 
       np.insert(np.delete(x.shape, split_axis), concat_axis, axis_size)
 
     where ``axis_size`` is the size of the mapped axis named ``axis_name`` in
     the input ``x``, i.e. ``axis_size = lax.psum(1, axis_name)``.
+
+    Otherwise array with shape similar to the input shape, except with split_axis
+    divided by axis size and concat_axis multiplied by axis size.
   """
-  def bind(x):
+  def bind(x, split_axis=split_axis, concat_axis=concat_axis):
     group_size = psum(1, axis_name, axis_index_groups=axis_index_groups)
-    if group_size != x.shape[split_axis]:
-      msg = ("all_to_all requires the size of the mapped axis axis_name to "
-             "equal x.shape[split_axis], but they are {} and {} respectively.")
-      raise ValueError(msg.format(group_size, x.shape[split_axis]))
-    return all_to_all_p.bind(x, split_axis=split_axis, concat_axis=concat_axis,
-                             axis_name=axis_name,
-                             axis_index_groups=axis_index_groups)
+    if tiled:
+      if x.shape[split_axis] % group_size != 0:
+        raise ValueError(f"The size of all_to_all split_axis ({x.shape[split_axis]}) "
+                         f"has to be divisible by the size of the named axis "
+                         f"{axis_name} ({group_size})")
+    else:
+      if group_size != x.shape[split_axis]:
+        msg = ("all_to_all requires the size of the mapped axis axis_name to "
+               "equal x.shape[split_axis], but they are {} and {} respectively.")
+        raise ValueError(msg.format(group_size, x.shape[split_axis]))
+      if split_axis < concat_axis:
+        concat_axis += 1  # concat_axis gives a position _after_ split_axis is removed
+        x = lax.expand_dims(x, (concat_axis,))  # insert the new axis
+      elif split_axis == concat_axis:
+        pass
+      else:  # concat_axis < split_axis
+        x = lax.expand_dims(x, (concat_axis,))  # insert the new axis
+        split_axis += 1   # we have a new axis before split_axis now
+    result = all_to_all_p.bind(x, split_axis=split_axis, concat_axis=concat_axis,
+                               axis_name=axis_name,
+                               axis_index_groups=axis_index_groups)
+    if not tiled and split_axis != concat_axis:
+      result = lax.squeeze(result, (split_axis,))
+    return result
 
   return tree_util.tree_map(bind, x)
 
@@ -602,10 +625,18 @@ def _allreduce_impl(pos_reducer, *args, axes, axis_index_groups):
   return [pos_reducer(arg, axes) for arg in args]
 
 def _allreduce_abstract_eval(*args, axes, axis_index_groups):
+  # TODO(frostig,mattjj,jekbradbury): maybe check aval names here
   pos_axes = tuple(axis for axis in axes if isinstance(axis, int))
+  named_shapes = [arg.named_shape for arg in args]
+  if axis_index_groups is None:
+    named_axes = set(axis for axis in axes if not isinstance(axis, int))
+    named_shapes = [{name: size for name, size in arg.named_shape.items()
+                     if name not in named_axes} for arg in args]
+  else:
+    assert len(pos_axes) == 0
   return [ShapedArray(lax._reduce_op_shape_rule(raise_to_shaped(arg), axes=pos_axes),
-                      arg.dtype)
-          for arg in args]
+                      arg.dtype, named_shape=named_shape)
+          for arg, named_shape in zip(args, named_shapes)]
 
 def _allreduce_translation_rule(prim, pos_prim, c, *args, axes, axis_index_groups,
                                 axis_env, platform):
@@ -813,13 +844,27 @@ def _moveaxis(src, dst, x):
   perm.insert(dst, src)
   return lax.transpose(x, perm)
 
+def _splitaxis(axis, factor, x):
+  new_shape = list(x.shape)
+  assert new_shape[axis] % factor == 0, (new_shape[axis], factor)
+  new_shape[axis:axis+1] = [factor, new_shape[axis] // factor]
+  return x.reshape(new_shape)
+
+def _foldaxis(axis, x):
+  new_shape = list(x.shape)
+  new_shape[axis:axis+2] = [x.shape[axis] * x.shape[axis + 1]]
+  return x.reshape(new_shape)
+
 def _all_to_all_via_all_gather(x, *, axis_name, split_axis, concat_axis, axis_index_groups):
-  global_full = all_gather(x, axis_name, axis_index_groups=axis_index_groups)
+  full = all_gather(x, axis_name, axis_index_groups=axis_index_groups)
   idx = axis_index(axis_name)
   if axis_index_groups:
     idx = idx % len(axis_index_groups[0])
-  local_slice = lax.dynamic_index_in_dim(global_full, idx, split_axis + 1, keepdims=False)
-  return _moveaxis(0, concat_axis, local_slice)
+  axis_size = full.shape[0]
+  tile_size = x.shape[split_axis] // axis_size
+  tile_base_idx = idx * tile_size
+  sliced = lax.dynamic_slice_in_dim(full, tile_base_idx, tile_size, split_axis + 1)
+  return _foldaxis(concat_axis, _moveaxis(0, concat_axis, sliced))
 
 def _all_to_all_translation_rule(c, x, *, split_axis, concat_axis, axis_name,
                                  axis_index_groups, axis_env, platform):
@@ -833,18 +878,7 @@ def _all_to_all_translation_rule(c, x, *, split_axis, concat_axis, axis_name,
     if not all(split_count == len(g) for g in replica_groups):
       raise ValueError('Replica groups must be equally sized')
     replica_groups_protos = xc.make_replica_groups(replica_groups)
-    if concat_axis == split_axis:
-      return xops.AllToAll(x, split_axis, concat_axis, split_count,
-                           replica_groups_protos)
-    else:
-      if concat_axis < split_axis:
-        split_axis += 1
-      elif split_axis < concat_axis:
-        concat_axis += 1
-      x = xla.lower_fun(partial(lax.expand_dims, dimensions=(concat_axis,)), multiple_results=False)(c, x)
-      x = xops.AllToAll(x, split_axis, concat_axis, split_count, replica_groups_protos)
-      x = xla.lower_fun(partial(lax.squeeze, dimensions=(split_axis,)), multiple_results=False)(c, x)
-      return x
+    return xops.AllToAll(x, split_axis, concat_axis, split_count, replica_groups_protos)
   else:
     warnings.warn(
         "all_to_all (and pswapaxes) are only implemented properly for TPUs and GPUs (if "
@@ -863,7 +897,6 @@ def _all_to_all_translation_rule(c, x, *, split_axis, concat_axis, axis_name,
         axis_env=axis_env,
         platform=platform)
 
-
 def _all_to_all_transpose_rule(cts, x, axis_name, split_axis, concat_axis, axis_index_groups):
   return (all_to_all(
       cts,
@@ -872,48 +905,74 @@ def _all_to_all_transpose_rule(cts, x, axis_name, split_axis, concat_axis, axis_
       concat_axis=split_axis,
       axis_index_groups=axis_index_groups),)
 
-
 def _all_to_all_batcher(vals_in, dims_in, *, axis_name, split_axis, concat_axis, axis_index_groups):
   x, = vals_in
   d, = dims_in
-  if d <= split_axis:
-    split_axis += 1
-  if d <= concat_axis:
-    concat_axis += 1
-  # Note: At this point split_axis and concat_axis are adjusted to the extra
-  #       dimension and we have d != split_axis and d != concat_axis.
-  if split_axis < d < concat_axis:
-    d -= 1
-  elif concat_axis < d < split_axis:
-    d += 1
   result = all_to_all_p.bind(
       x,
       axis_name=axis_name,
-      split_axis=split_axis,
-      concat_axis=concat_axis,
+      split_axis=split_axis + (d <= split_axis),
+      concat_axis=concat_axis + (d <= concat_axis),
       axis_index_groups=axis_index_groups)
   return result, d
 
 def _all_to_all_batched_collective(frame, vals_in, dims_in,
                                    axis_name, split_axis, concat_axis,
                                    axis_index_groups):
-  if isinstance(axis_name, (list, tuple)) and len(axis_name) > 1:
-    raise NotImplementedError("update after #4835")  # TODO(mattjj,apaszke)
+  if axis_index_groups is not None:
+    raise NotImplementedError("Please open a feature request!")
   x, = vals_in
   d, = dims_in
-  split_axis_adj = split_axis + (1 if d <= split_axis else 0)
-  concat_axis_adj = concat_axis + (1 if split_axis_adj <= concat_axis else 0)
-  if d < split_axis_adj < concat_axis_adj:
-    split_axis_adj -= 1
-  elif concat_axis_adj < split_axis_adj < d:
-    split_axis_adj += 1
-  return _moveaxis(d, concat_axis_adj, x), split_axis_adj
+  if isinstance(axis_name, (list, tuple)):
+    pos = axis_name.index(frame.name)
+    major_axes, minor_axes = axis_name[:pos], axis_name[pos + 1:]
+  else:
+    major_axes, minor_axes = (), ()
+  # Optimized case when no splitting is necessary
+  if not major_axes and not minor_axes:
+    if split_axis == concat_axis:
+      axis = split_axis + (d <= split_axis)
+      d_pre_split = d
+      x = _splitaxis(axis, frame.size, x)
+      d += (axis <= d)
+      return _foldaxis(axis, moveaxis(x, (d, axis), (axis, d))), d_pre_split
+    else:
+      x_concat = _foldaxis(concat_axis, _moveaxis(d, concat_axis, x))
+      return _splitaxis(split_axis, frame.size, x_concat), split_axis
+  # Here we have to handle either the major or the minor dimensions
+  # We will be accumulating chunks into the three leading dims: [Major, Current, Minor, ...]
+  x, d = lax.expand_dims(_moveaxis(d, 0, x), (0, 2)), 1
+  split_axis += 3; concat_axis += 3  # Offset by extra three leading dims
+
+  if major_axes:
+    x = all_to_all_p.bind(x, axis_name=major_axes,
+                          split_axis=split_axis, concat_axis=0,
+                          axis_index_groups=axis_index_groups)
+  # Split out the local part into axis new_d (NOTE: d is already in axis 1)
+  x = _splitaxis(split_axis, frame.size, x)
+  new_d = split_axis
+  concat_axis += (split_axis <= concat_axis)  # Offset the existing axes by the new batch axis
+  split_axis += 1
+  if minor_axes:
+    x = all_to_all_p.bind(x, axis_name=minor_axes,
+                          split_axis=split_axis, concat_axis=2,
+                          axis_index_groups=axis_index_groups)
+
+  # Fold the chunk axes into a single one
+  x = _foldaxis(0, _foldaxis(0, x))
+  split_axis -= 2; concat_axis -= 2; new_d -= 2
+  # Fold gathered axes into concat_axis
+  x = _foldaxis(concat_axis - 1, _moveaxis(0, concat_axis - 1, x))
+  new_d -= 1  # We've removed 0th dimension, so new_d needs to be adjusted
+  return x, new_d
 
 def _all_to_all_abstract_eval(x, axis_name, split_axis, concat_axis, axis_index_groups):
   input_aval = raise_to_shaped(x)
   shape = list(input_aval.shape)
-  size = shape.pop(split_axis)
-  shape.insert(concat_axis, size)
+  axis_size = psum(1, axis_name) if axis_index_groups is None else len(axis_index_groups[0])
+  assert shape[split_axis] % axis_size == 0, (shape[split_axis], axis_size)
+  shape[split_axis] //= axis_size
+  shape[concat_axis] *= axis_size
   return input_aval.update(shape=tuple(shape), weak_type=False)
 
 all_to_all_p = core.Primitive('all_to_all')
@@ -1031,18 +1090,16 @@ def _all_gather_abstract_eval(x, *, all_gather_dimension, axis_name, axis_index_
   x_aval = raise_to_shaped(x)
   new_shape = list(x_aval.shape)
   new_shape.insert(all_gather_dimension, axis_size)
-  return x_aval.update(shape=new_shape)
+  new_named_shape = {name: size for name, size in x_aval.named_shape.items()
+                     if name != axis_name}
+  return x_aval.update(shape=new_shape, named_shape=new_named_shape)
 
 def _all_gather_transpose_rule(cts, x, *, all_gather_dimension, axis_name, axis_index_groups, axis_size):
   # TODO(cjfj): Add reduce-scatter op to XLA?
   concat_axis = 0
-  return (lax_numpy.sum(
-      all_to_all(
-          cts,
-          axis_name=axis_name,
-          split_axis=all_gather_dimension,
-          concat_axis=concat_axis,
-          axis_index_groups=axis_index_groups),
+  return (lax_numpy.sum(all_to_all(
+      cts, axis_name=axis_name, split_axis=all_gather_dimension,
+      concat_axis=concat_axis, axis_index_groups=axis_index_groups),
       axis=concat_axis),)
 
 def _all_gather_batcher(vals_in, dims_in, *, all_gather_dimension, axis_name, axis_index_groups, axis_size):
@@ -1086,10 +1143,13 @@ def _axis_index_translation_rule(c, *, axis_name, axis_env, platform):
   unsigned_index = xops.Rem(xops.Div(xops.ReplicaId(c), div), mod)
   return xops.ConvertElementType(unsigned_index, xb.dtype_to_etype(np.int32))
 
+def _axis_index_abstract_eval(*, axis_name):
+  frame = core.axis_frame(axis_name)
+  return ShapedArray((), np.int32, named_shape={axis_name: frame.size})
+
 axis_index_p = core.Primitive('axis_index')
 xla.parallel_translations[axis_index_p] = _axis_index_translation_rule
-axis_index_p.def_abstract_eval(
-    lambda *args, **params: ShapedArray((), np.int32))
+axis_index_p.def_abstract_eval(_axis_index_abstract_eval)
 pxla.multi_host_supported_collectives.add(axis_index_p)
 core.axis_substitution_rules[axis_index_p] = partial(_subst_all_names_in_param, 'axis_name')
 
@@ -1098,26 +1158,30 @@ core.axis_substitution_rules[axis_index_p] = partial(_subst_all_names_in_param,
 # wants to bind an axis name has to additionally implement `process_axis_index`
 # and put its main trace on the axis env stack.
 def _axis_index_bind(*, axis_name):
-  if not isinstance(axis_name, (tuple, list)):
-    axis_name = (axis_name,)
-  inner_size = 1
-  index = 0
-  for name in reversed(axis_name):
+  def name_idx(name):
     frame = core.axis_frame(name)
-    if frame.main_trace is not None:
-      trace = frame.main_trace.with_cur_sublevel()
-      name_idx = trace.process_axis_index(frame)
+    dynamic = core.thread_local_state.trace_state.trace_stack.dynamic
+    if (frame.main_trace is None or dynamic.level > frame.main_trace.level):
+      return core.Primitive.bind(axis_index_p, axis_name=name)
     else:
-      name_idx = core.Primitive.bind(axis_index_p, axis_name=name)
-    index += name_idx * inner_size
-    inner_size *= psum(1, name)
-  return index
+      trace = frame.main_trace.with_cur_sublevel()
+      return trace.process_axis_index(frame)
+
+  if not isinstance(axis_name, (tuple, list)):
+    return name_idx(axis_name)
+  else:
+    inner_size = 1
+    index = 0
+    for name in reversed(axis_name):
+      index += name_idx(name) * inner_size
+      inner_size *= psum(1, name)
+    return index
 axis_index_p.def_custom_bind(_axis_index_bind)
 
-def _process_axis_index(self, frame):
+def _vmap_process_axis_index(self, frame):
   assert frame.size is not None
-  return batching.BatchTracer(self, lax_numpy.arange(frame.size, dtype=np.int32), 0)
-batching.BatchTrace.process_axis_index = _process_axis_index  # type: ignore
+  return batching.BatchTracer(self, lax.iota(np.int32, frame.size), 0)
+batching.BatchTrace.process_axis_index = _vmap_process_axis_index  # type: ignore
 
 
 pdot_p = core.Primitive('pdot')
@@ -1130,11 +1194,15 @@ def _pdot_impl(x, y, *, axis_name, pos_contract, pos_batch):
 
 @pdot_p.def_abstract_eval
 def _pdot_abstract_eval(x, y, *, axis_name, pos_contract, pos_batch):
-  # TODO: avals with names, check inputs are mapped along axis_name, eliminate
+  # TODO(frostig,mattjj,jekbradbury): check inputs have given axis names?
   if not len(set(axis_name)) == len(axis_name): raise ValueError
-  return lax.dot_general_p.abstract_eval(
+  pos_aval = lax.dot_general_p.abstract_eval(
       x, y, dimension_numbers=[pos_contract, pos_batch],
       precision=None, preferred_element_type=None)
+  named_shape = {name: size
+                 for aval in (x, y) for name, size in aval.named_shape.items()
+                 if name not in axis_name}
+  return pos_aval.update(named_shape=named_shape)
 
 def _pdot_vmap_collective_rule(frame, vals_in, dims_in, *, axis_name,
                                pos_contract, pos_batch):
@@ -1286,7 +1354,8 @@ def omnistaging_disabler() -> None:
     nreps = dynamic_axis_env.nreps
     trace = frame.pmap_trace
 
-    out_aval = ShapedArray((), np.int32)
+    out_aval = _axis_index_abstract_eval(
+        nreps=nreps, sizes=sizes, axis_name=axis_name)
     out_tracer = pe.JaxprTracer(trace, pe.PartialVal.unknown(out_aval), None)
     eqn = pe.new_eqn_recipe([], [out_tracer], axis_index_p,
                             dict(nreps=nreps, sizes=sizes, axis_name=axis_name),
@@ -1301,7 +1370,9 @@ def omnistaging_disabler() -> None:
     unsigned_index = xops.Rem(xops.Div(xops.ReplicaId(c), div), mod)
     return xops.ConvertElementType(unsigned_index, xb.dtype_to_etype(np.int32))
 
+  def _axis_index_abstract_eval(*, nreps, sizes, axis_name):
+    return ShapedArray((), np.int32, named_shape={axis_name: sizes[-1]})
+
   axis_index_p.def_custom_bind(_axis_index_bind)
-  axis_index_p.def_abstract_eval(
-      lambda *args, **params: ShapedArray((), np.int32))
+  axis_index_p.def_abstract_eval(_axis_index_abstract_eval)
   xla.translations[axis_index_p] = _axis_index_translation_rule
