@@ -18,7 +18,7 @@ import scipy.stats as osp_stats
 from jax import lax
 from jax._src.numpy.util import _wraps
 from jax._src.numpy import lax_numpy as jnp
-from jax.scipy.special import xlogy, gammaln
+from jax.scipy.special import xlogy, gammaln, gammaincc
 
 
 @_wraps(osp_stats.poisson.logpmf, update_doc=False)
@@ -32,3 +32,11 @@ def logpmf(k, mu, loc=0):
 @_wraps(osp_stats.poisson.pmf, update_doc=False)
 def pmf(k, mu, loc=0):
   return jnp.exp(logpmf(k, mu, loc))
+
+@_wraps(osp_stats.poisson.cdf, update_doc=False)
+def cdf(k, mu, loc=0):
+  k, mu, loc = jnp._promote_args_inexact("poisson.logpmf", k, mu, loc)
+  zero = jnp._constant_like(k, 0)
+  x = lax.sub(k, loc)
+  p = gammaincc(jnp.floor(1 + x), mu)
+  return jnp.where(lax.lt(x, zero), zero, p)
