@@ -15,6 +15,10 @@
 import operator
 from typing import Any, Tuple, Union
 
+import numpy as np
+
+from . import core
+from . import dtypes
 from .tree_util import (tree_flatten, tree_unflatten, tree_multimap, _replace_nones,
                         tree_structure, treedef_children, treedef_is_leaf)
 from . import linear_util as lu
@@ -71,7 +75,11 @@ def apply_flat_fun_nokwargs(fun, io_tree, py_args):
 @lu.transformation_with_aux
 def flatten_fun_nokwargs2(in_tree, *args_flat):
   py_args = tree_unflatten(in_tree, args_flat)
-  ans, aux = yield py_args, {}
+  pair = yield py_args, {}
+  if not isinstance(pair, (list, tuple)) or len(pair) != 2:
+    raise TypeError("expected function with aux output to return a two-element "
+                    f"tuple, but got type {type(pair)} with value {repr(pair)}")
+  ans, aux = pair
   ans_flat, ans_tree = tree_flatten(ans)
   aux_flat, aux_tree = tree_flatten(aux)
   yield (ans_flat, aux_flat), (ans_tree, aux_tree)
@@ -191,3 +199,20 @@ def flatten_axes(name, treedef, axis_tree, *, kws=False):
   axes = [None if a is proxy else a for a in axes]
   assert len(axes) == treedef.num_leaves
   return axes
+
+def _dtype(x):
+  try:
+    return dtypes.result_type(x)
+  except ValueError:
+    return dtypes.result_type(getattr(x, 'dtype'))
+
+def shaped_abstractify(x):
+  try:
+    return core.raise_to_shaped(core.get_aval(x))
+  except TypeError:
+    pass
+
+  weak_type = getattr(x, 'weak_type', False)
+  named_shape = getattr(x, 'named_shape', {})
+  return core.ShapedArray(np.shape(x), _dtype(x), weak_type=weak_type,
+                          named_shape=named_shape)
