@@ -34,6 +34,8 @@ from .errors import (ConcretizationTypeError, TracerArrayConversionError,
                      TracerIntegerConversionError)
 from . import linear_util as lu
 
+from . import lib
+from .lib import jax_jit
 from jax._src import source_info_util
 from ._src.util import (safe_zip, safe_map, partial, curry, prod, partialmethod,
                    tuple_insert, tuple_delete, as_hashable_function, unzip2,
@@ -43,33 +45,6 @@ from ._src.pprint_util import pp, vcat, PrettyPrint
 from ._src import traceback_util
 traceback_util.register_exclusion(__file__)
 
-# TODO(mattjj): move this into debug_state
-skip_checks = not FLAGS.jax_enable_checks
-
-@contextmanager
-def skipping_checks():
-  """Context manager for temporarily disabling internal checks."""
-  global skip_checks
-  old_value, skip_checks = skip_checks, True
-  try:
-    yield
-  finally:
-    skip_checks = old_value
-
-@contextmanager
-def checking_leaks():
-  """Context manager for temporarily enabling tracer leak checks."""
-  old_value, debug_state.check_leaks = debug_state.check_leaks, True
-  try:
-    yield
-  finally:
-    debug_state.check_leaks = old_value
-
-class DebugState(threading.local):
-  def __init__(self):
-    self.check_leaks = FLAGS.jax_check_tracer_leaks
-debug_state = DebugState()
-
 zip = safe_zip
 map = safe_map
 
@@ -277,8 +252,8 @@ class Primitive:
 
 
   def bind(self, *args, **params):
-    assert skip_checks or all(isinstance(arg, Tracer)
-                              or valid_jaxtype(arg) for arg in args), args
+    assert (not config.jax_enable_checks or
+            all(isinstance(arg, Tracer) or valid_jaxtype(arg) for arg in args)), args
     top_trace = find_top_trace(args)
     tracers = map(top_trace.full_raise, args)
     out = top_trace.process_primitive(self, tracers, params)
@@ -567,7 +542,7 @@ class Tracer:
 
   def __getattr__(self, name):
     # if the aval property raises an AttributeError, gets caught here
-    assert skip_checks or name != "aval"
+    assert not config.jax_enable_checks or name != "aval"
 
     try:
       attr = getattr(self.aval, name)
@@ -688,7 +663,23 @@ class TraceStack:
     new.dynamic = self.dynamic
     return new
 
-class Sublevel(int): pass
+
+@total_ordering
+class Sublevel:
+
+  def __init__(self, level: int):
+    self.level = level
+
+  def __repr__(self):
+    return str(self.level)
+
+  def __eq__(self, other):
+    return type(other) is Sublevel and self.level == other.level
+
+  def __lt__(self, other):
+    return type(other) is Sublevel and self.level < other.level
+
+
 AxisEnvFrame = namedtuple('AxisEnvFrame', ['name', 'size', 'main_trace'])
 AxisName = Hashable
 
@@ -709,12 +700,18 @@ class TraceState:
     new.axis_env = self.axis_env[:]
     return new
 
+def extra_jit_context(trace_stack):
+    return trace_stack.dynamic
+
 # The global state of the tracer is accessed by a thread-local object.
 # This allows concurrent tracing in separate threads; passing traced objects
 # between threads is forbidden.
 class ThreadLocalState(threading.local):
   def __init__(self):
     self.trace_state = TraceState()
+    if lib._xla_extension_version >= 11:
+      jax_jit.thread_local_state().extra_jit_context = extra_jit_context(
+          self.trace_state.trace_stack)
 thread_local_state = ThreadLocalState()
 
 def trace_state_clean() -> bool:
@@ -745,7 +742,10 @@ def new_main(trace_type: Type[Trace],
   main = MainTrace(level, trace_type, **payload)
   stack.push(main)
   if dynamic:
+    jit_tls = jax_jit.thread_local_state()
     prev_dynamic, stack.dynamic = stack.dynamic, main
+    if lib._xla_extension_version >= 11:
+      jit_tls.extra_jit_context = extra_jit_context(stack)
 
   try:
     yield main
@@ -753,8 +753,10 @@ def new_main(trace_type: Type[Trace],
     stack.pop()
     if dynamic:
       stack.dynamic = prev_dynamic
+      if lib._xla_extension_version >= 11:
+        jit_tls.extra_jit_context = extra_jit_context(stack)
 
-  if debug_state.check_leaks:
+  if config.jax_check_tracer_leaks:
     t = ref(main)
     del main
     if t() is not None:
@@ -767,13 +769,18 @@ def new_base_main(trace_type: Type[Trace]) -> Generator[MainTrace, None, None]:
   main = MainTrace(0, trace_type)
   prev_dynamic, stack.dynamic = stack.dynamic, main
   prev_base, stack.stack[0] = stack.stack[0], main
+  jit_tls = jax_jit.thread_local_state()
+  if lib._xla_extension_version >= 11:
+    jit_tls.extra_jit_context = extra_jit_context(stack)
   try:
     yield main
   finally:
     stack.dynamic = prev_dynamic
     stack.stack[0] = prev_base
+    if lib._xla_extension_version >= 11:
+      jit_tls.extra_jit_context = extra_jit_context(stack)
 
-  if debug_state.check_leaks:
+  if config.jax_check_tracer_leaks:
     t = ref(main)
     del main
     if t() is not None:
@@ -793,12 +800,11 @@ def new_sublevel() -> Generator[None, None, None]:
   finally:
     thread_local_state.trace_state.substack.pop()
 
-  # TODO(mattjj): to check sublevel leaks, we need to make Sublevel weakref-able
-  # if debug_state.check_leaks:
-  #   t = ref(sublevel)
-  #   del sublevel
-  #   if t() is not None:
-  #     raise Exception('Leaked sublevel {}'.format(t()))
+  if config.jax_check_tracer_leaks:
+    t = ref(sublevel)
+    del sublevel
+    if t() is not None:
+      raise Exception(f'Leaked sublevel {t()}.')
 
 def maybe_new_sublevel(trace):
   # dynamic traces run the WrappedFun, so we raise the sublevel for them
@@ -866,7 +872,7 @@ class AbstractUnit(AbstractValue):
   # _num_buffers = 0
   def at_least_vspace(self): return self
   def join(self, other):
-    if not skip_checks:
+    if config.jax_enable_checks:
       assert other is abstract_unit, other
     return self
   def _eq(self, self_traced, other): return get_aval(other) is self
@@ -963,6 +969,8 @@ def concrete_or_error(force: Any, val: Any, context=""):
   else:
     return force(val)
 
+convert_element_type_p = Primitive('convert_element_type')
+
 class UnshapedArray(AbstractValue):
   __slots__ = ['dtype', 'weak_type']
   array_abstraction_level = 2
@@ -1529,7 +1537,31 @@ def used_axis_names(primitive: Primitive, params: ParamDict) -> Set[AxisName]:
 def subst_axis_names(primitive: Primitive, params: ParamDict, subst: AxisSubst) -> ParamDict:
   if primitive in axis_substitution_rules:
     return axis_substitution_rules[primitive](params, subst)
-  return params
+  # Default implementation: substitute names in all jaxpr parameters
+  if isinstance(primitive, MapPrimitive):
+    def shadowed_subst(name):
+      return (name,) if name == params['axis_name'] else subst(name)
+  else:
+    shadowed_subst = subst
+  jaxpr_params = [(n, v) for n, v in params.items() if isinstance(v, (Jaxpr, ClosedJaxpr))]
+  if not jaxpr_params:
+    return params
+  new_params = dict(params)
+  for name, jaxpr in jaxpr_params:
+    new_params[name] = subst_axis_names_jaxpr(jaxpr, shadowed_subst)
+  return new_params
+
+def subst_axis_names_jaxpr(jaxpr: Union[Jaxpr, ClosedJaxpr], subst: AxisSubst):
+  consts = None
+  if isinstance(jaxpr, ClosedJaxpr):
+    consts = jaxpr.consts
+    jaxpr = jaxpr.jaxpr
+  eqns = [eqn._replace(params=subst_axis_names(eqn.primitive, eqn.params, subst))
+          for eqn in jaxpr.eqns]
+  new_jaxpr = Jaxpr(jaxpr.constvars, jaxpr.invars, jaxpr.outvars, eqns)
+  if consts is not None:
+    return ClosedJaxpr(new_jaxpr, consts)
+  return new_jaxpr
 
 axis_substitution_rules: Dict[Primitive, Callable[[ParamDict, AxisSubst], ParamDict]] = {}
 
@@ -1698,7 +1730,8 @@ def check_map(prim, in_avals, params):
   mapped_avals = [mapped_aval(axis_size, in_axis, aval)
                   if in_axis is not None else aval
                   for aval, in_axis in zip(in_avals, in_axes)]
-  _check_jaxpr(call_jaxpr, mapped_avals)
+  with extend_axis_env(params['axis_name'], axis_size, None):
+    _check_jaxpr(call_jaxpr, mapped_avals)
 
   mapped_out_avals = [v.aval for v in call_jaxpr.outvars]
   out_avals = [unmapped_aval(axis_size, out_axis, aval) if out_axis is not None else aval
@@ -1790,7 +1823,7 @@ def pp_kv_pairs(kv_pairs):
 def omnistaging_disabler() -> None:
   global thread_local_state, call_bind, find_top_trace, initial_style_staging, \
       new_main, reset_trace_state, TraceStack, TraceState, extend_axis_env, \
-      eval_context
+      eval_context, extra_jit_context
 
   class TraceStack:
     upward: List[MainTrace]
@@ -1846,6 +1879,9 @@ def omnistaging_disabler() -> None:
       new.initial_style = self.initial_style
       return new
 
+  def extra_jit_context(trace_stack):
+    return None
+
   thread_local_state = ThreadLocalState()
 
   def reset_trace_state() -> bool:
@@ -1869,7 +1905,7 @@ def omnistaging_disabler() -> None:
     finally:
       thread_local_state.trace_state.trace_stack.pop(bottom)
 
-    if debug_state.check_leaks:
+    if config.jax_check_tracer_leaks:
       t = ref(main)
       del main
       if t() is not None:
@@ -1886,7 +1922,7 @@ def omnistaging_disabler() -> None:
     yield  # dummy implementation for forward compatibility
 
   def bind(self, *args, **kwargs):
-    assert skip_checks or all(isinstance(arg, Tracer)
+    assert not config.jax_enable_checks or all(isinstance(arg, Tracer)
                               or valid_jaxtype(arg) for arg in args), args
     top_trace = find_top_trace(args)
     if top_trace is None:
