@@ -70,9 +70,9 @@ def _is_tfval(v: TfVal) -> bool:
   if isinstance(v, (tf.Tensor, tf.Variable)):
     return True
   try:
-    # Note: this conversion is overkill and just intended as a type check; this code
-    # is in principle only run if core.skip_checks is False.
-    # TODO: it is not true that this code is run only without skip_checks
+    # Note: this conversion is overkill and just intended as a type check; this
+    # code is in principle only run if config.jax_enable_checks is True.
+    # TODO: it is not true that this code is run only with jax_enable_checks.
     _safe_convert_to_tensor(v)
     return True
   except ValueError:
@@ -130,37 +130,7 @@ def convert(fun: Callable, *,
       JAX arrays, or (nested) standard Python containers (tuple/list/dict)
       thereof.
 
-    in_shapes: an optional sequence of shape specifications,
-      one for each argument of the function to be converted. Default is a
-      list of `None`, in which case the argument shape specifications are taken
-      from the shapes of the actual arguments.
-
-      A non-default `in_shapes` is needed sometimes when the actual arguments
-      have partially-specified shapes. If an argument is a pytree, then the
-      shape specification must be a matching pytree or `None`.
-      See [how optional parameters are matched to arguments](https://jax.readthedocs.io/en/latest/pytrees.html#applying-optional-parameters-to-pytrees).
-      A shape specification should be a string, with comma-separated dimension
-      specifications, and optionally wrapped in parentheses. A dimension
-      specification is either a number, or the placeholder `_`, or a lowercase
-      word denoting a name for a dimension variable, or an arithmetic expression
-      with integer literals, dimension variables, and the operators `+` and `*`.
-      In presence of dimension variables, the conversion is done with a
-      shape abstraction that allows any concrete value for the variable.
-
-      Examples of shape specifications:
-        * `[None, "(batch, 16)"]`: no specification for the first argument (takes
-          the shape from the actual argument); the second argument is a 2D
-          array with the first dimension size set to a variable `batch` and the
-          second dimension 16.
-        * `["(batch, _)", "(batch,)"]`: the leading dimensions of the two arguments
-          must match. The second dimension of the first argument is taken from the
-          actual argument shape.
-        * `[(batch, 2 * batch)]`: a 2D matrix with the second dimension having
-          double the size of the first one.
-
-      See [the README](https://github.com/google/jax/blob/master/jax/experimental/jax2tf/README.md#shape-polymorphic-conversion)
-      for more details.
-
+    in_shapes: Not used. This is kept for backwards compatibility (see #6080).
     with_gradient: if set, will add a tf.custom_gradient to the converted
       function, by converting the ``jax.vjp(fun)``. Only first-order
       differentiation is supported for now. If the converted function is
@@ -212,11 +182,8 @@ def convert(fun: Callable, *,
     if in_shapes is None:
       in_shapes_ = (None,) * len(args)
     else:
-      if not isinstance(in_shapes, Sequence) or len(args) != len(in_shapes):
-        msg = ("in_shapes must be a sequence as long as the argument list "
-               f"({len(args)}). Got in_shapes={in_shapes}.")
-        raise TypeError(msg)
-      in_shapes_ = tuple(in_shapes)
+      raise ValueError("The `in_shapes` parameter is not supported")
+
     # Expand the in_shapes to match the argument pytree
     in_shapes_flat = tuple(api_util.flatten_axes("jax2tf.convert in_shapes",
                                                  in_tree.children()[0], in_shapes_))
@@ -386,7 +353,7 @@ def _tfval_shape_dtype(val: TfVal) -> Tuple[Sequence[Optional[int]], DType]:
     # May be partially known
     return tuple(val.shape), to_jax_dtype(val.dtype)
   else:  # Must be a numeric value
-    assert core.skip_checks or _is_tfval(val), f"Non TfVal: {val}"
+    assert not config.jax_enable_checks or _is_tfval(val), f"Non TfVal: {val}"
     raw_aval = xla.abstractify(val)
     return raw_aval.shape, raw_aval.dtype  # type: ignore[attr-defined]
 
@@ -638,7 +605,7 @@ class TensorFlowTracer(core.Tracer):
         val = tf.cast(val, dtype=aval_dtype)
         val_dtype = aval_dtype
 
-      if not core.skip_checks:
+      if config.jax_enable_checks:
         assert aval_dtype == val_dtype, f"expected {aval_dtype} == {val_dtype}"
         for aval_dim, val_dim in util.safe_zip(self._aval.shape, val_shape):  # type: ignore[attr-defined]
           if val_dim is None:
@@ -736,7 +703,7 @@ class TensorFlowTrace(core.Trace):
 
     # Check that the impl rule returned a value of expected shape and dtype
     # TODO: adapt this to match polymorphic shapes
-    if not core.skip_checks:
+    if config.jax_enable_checks:
       if primitive.multiple_results:
         for o, expected_aval in zip(out, out_aval):  # type: ignore
           assert o.aval.strip_weak_type() == expected_aval.strip_weak_type(), (
@@ -830,7 +797,7 @@ for unexpected in xla.call_translations: # Call primitives are inlined
 
 # Primitives that are not yet implemented must be explicitly declared here.
 tf_not_yet_impl = [
-  "reduce", "rng_uniform",
+  "reduce", "rng_uniform", "clz",
 
   "igamma_grad_a",
   "random_gamma_grad",
@@ -840,6 +807,7 @@ tf_not_yet_impl = [
   "infeed", "outfeed", "pmax_p",
   "pmin", "ppermute", "psum", "pmax", "pgather",
   "axis_index", "pdot", "all_gather",
+  "rng_bit_generator",
 
   "xla_pmap",
   "call_tf",
@@ -1562,7 +1530,7 @@ def _common_reduce_window(operand, init_val, reducer, window_dimensions,
   reducer_fn = tf.function(reducer, autograph=False).get_concrete_function(o_spec, o_spec)
 
   if not isinstance(init_val, tf.Tensor):
-    assert core.skip_checks or _is_tfval(init_val), f"Non TfVal: {init_val}"
+    assert not config.jax_enable_checks or _is_tfval(init_val), f"Non TfVal: {init_val}"
     init_val = tf.constant(init_val, operand.dtype)
   out = tfxla.reduce_window(operand, init_val,
                             reducer_fn, window_dimensions,
