@@ -135,7 +135,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
          dtype=dtype)
     for dtype in [np.int64, np.float64]))
   def test_converts_64bit(self, dtype=np.int64, with_function=False):
-    if not config.FLAGS.jax_enable_x64:
+    if not config.jax_enable_x64:
       self.skipTest("requires x64 mode")
     big_const = np.full((5,), 2 ** 33, dtype=dtype)
     self.ConvertAndCompare(jnp.sin, big_const)
