@@ -14,19 +14,21 @@
 
 import contextlib
 import logging
-import numpy as np
+
 from typing import Any, Callable, List, Optional, Sequence
-import tensorflow as tf  # type: ignore[import]
+
 
 import jax
-from jax.config import config
 from jax import dtypes
-from jax.experimental import jax2tf
-from jax.interpreters import masking
+from jax import numpy as jnp
 from jax import test_util as jtu
 from jax import tree_util
-from jax import numpy as jnp
 
+from jax.config import config
+from jax.experimental import jax2tf
+from jax.interpreters import masking
+import numpy as np
+import tensorflow as tf  # type: ignore[import]
 
 DType = Any
 
@@ -74,18 +76,15 @@ class JaxToTfTestCase(jtu.JaxTestCase):
   def setUp(self):
     super().setUp()
     # Ensure that all TF ops are created on the proper device (TPU or GPU or CPU)
-    # TODO(necula): why doesn't TF do this automatically?
     tf_preferred_devices = (
         tf.config.list_logical_devices("TPU") +
         tf.config.list_logical_devices("GPU") +
         tf.config.list_logical_devices())
     self.tf_default_device = tf_preferred_devices[0]
     logging.info(f"Running jax2tf converted code on {self.tf_default_device}.")
-    if jtu.device_under_test() != "gpu":
-      # TODO(necula): Change the build flags to ensure the GPU is seen by TF
-      # It seems that we need --config=cuda build flag for this to work?
-      self.assertEqual(jtu.device_under_test().upper(),
-                       self.tf_default_device.device_type)
+    # We need --config=cuda build flag for TF to see the GPUs
+    self.assertEqual(jtu.device_under_test().upper(),
+                     self.tf_default_device.device_type)
 
     with contextlib.ExitStack() as stack:
       stack.enter_context(tf.device(self.tf_default_device))
@@ -97,7 +96,7 @@ class JaxToTfTestCase(jtu.JaxTestCase):
     def to_numpy_dtype(dt):
       return dt if isinstance(dt, np.dtype) else dt.as_numpy_dtype
 
-    if not config.FLAGS.jax_enable_x64 and canonicalize_dtypes:
+    if not config.x64_enabled and canonicalize_dtypes:
       self.assertEqual(
           dtypes.canonicalize_dtype(to_numpy_dtype(jtu._dtype(x))),
           dtypes.canonicalize_dtype(to_numpy_dtype(jtu._dtype(y))))
@@ -201,7 +200,7 @@ class JaxToTfTestCase(jtu.JaxTestCase):
              "successful modes:\n" + "\n".join(unexpected_successes))
       logging.warning(msg)
       # Uncomment the below if you want to see warnings as failures
-      #self.assertEmpty(msg)
+      # self.assertEmpty(msg)
     return result_jax, result_tf
 
   def TransformConvertAndCompare(self, func: Callable, arg,
