@@ -49,7 +49,7 @@ class PartialVal(tuple):
   """
   def __new__(cls, xs: Tuple[Optional[AbstractValue], core.Value]):
     pv, const = xs
-    if not core.skip_checks:
+    if config.jax_enable_checks:
       # type checks
       assert isinstance(pv, (AbstractValue, type(None))), xs
       assert isinstance(const, core.Tracer) or type(const) is Zero or core.valid_jaxtype(const), xs
@@ -172,7 +172,9 @@ class JaxprTrace(Trace):
       return call_partial_eval_rules[primitive](self, primitive, f, tracers, params)
 
     in_pvals = [t.pval for t in tracers]
+    ctx: Any
     if primitive.map_primitive:
+      ctx = core.extend_axis_env(params['axis_name'], params['axis_size'], None)
       mapped_aval = partial(core.mapped_aval, params['axis_size'])
       in_pvals = [pval if pval.is_known() or in_axis is None
                   else PartialVal.unknown(mapped_aval(in_axis, pval[0]))
@@ -188,51 +190,53 @@ class JaxprTrace(Trace):
         pe_params = dict(params, out_axes_thunk=new_out_axes_thunk)
         return primitive.bind(f, *args, **pe_params)
     else:
+      ctx = contextlib.suppress()  # This is a no-op
       app = partial(primitive.bind, **params)
-    jaxpr, out_pvals, consts, env_tracers = self.partial_eval(
-        f, in_pvals, app, instantiate=False)
-    if primitive.map_primitive:
-      unmapped_aval = partial(core.unmapped_aval, params['axis_size'])
-      out_axes = params['out_axes_thunk']()
-      out_pvals = [pval if pval.is_known() else
-                   PartialVal.unknown(unmapped_aval(out_axis, pval[0])) if out_axis is not None else
-                   PartialVal.unknown(pval[0])
-                   for pval, out_axis in zip(out_pvals, out_axes)]
+    with ctx:
+      jaxpr, out_pvals, consts, env_tracers = self.partial_eval(
+          f, in_pvals, app, instantiate=False)
+      if primitive.map_primitive:
+        unmapped_aval = partial(core.unmapped_aval, params['axis_size'])
+        out_axes = params['out_axes_thunk']()
+        out_pvals = [pval if pval.is_known() else
+                     PartialVal.unknown(unmapped_aval(out_axis, pval[0])) if out_axis is not None else
+                     PartialVal.unknown(pval[0])
+                     for pval, out_axis in zip(out_pvals, out_axes)]
 
-    # Skip known invars and outvars, and lift constants as regular invars
-    in_knowns = tuple(t.pval.is_known() for t in it.chain(env_tracers, tracers))
-    out_unknowns = tuple(not pval.is_known() for pval in out_pvals)
-    jaxpr = _drop_invars(jaxpr, in_knowns)
-    jaxpr = _dce_open_jaxpr(jaxpr, out_unknowns, drop_outputs=True)
+      # Skip known invars and outvars, and lift constants as regular invars
+      in_knowns = tuple(t.pval.is_known() for t in it.chain(env_tracers, tracers))
+      out_unknowns = tuple(not pval.is_known() for pval in out_pvals)
+      jaxpr = _drop_invars(jaxpr, in_knowns)
+      jaxpr = _dce_open_jaxpr(jaxpr, out_unknowns, drop_outputs=True)
 
-    # Known tracers get propagated as if they were constants
-    known_tracers_out = [self.new_const(pval.get_known()) for pval in out_pvals
-                         if pval.is_known()]
+      # Known tracers get propagated as if they were constants
+      known_tracers_out = [self.new_const(pval.get_known()) for pval in out_pvals
+                           if pval.is_known()]
 
-    # Unknown tracers need to have the jaxpr set up as their recipe
-    unknown_tracers_out = [JaxprTracer(self, pval, None) for pval in out_pvals
-                           if not pval.is_known()]
-    unknown_tracers_in = [t for t in tracers if not t.pval.is_known()]
-    const_tracers = map(self.new_instantiated_const, consts)
-    in_tracers = (*const_tracers, *env_tracers, *unknown_tracers_in)
+      # Unknown tracers need to have the jaxpr set up as their recipe
+      unknown_tracers_out = [JaxprTracer(self, pval, None) for pval in out_pvals
+                             if not pval.is_known()]
+      unknown_tracers_in = [t for t in tracers if not t.pval.is_known()]
+      const_tracers = map(self.new_instantiated_const, consts)
+      in_tracers = (*const_tracers, *env_tracers, *unknown_tracers_in)
 
-    # Set up new params
-    new_params = dict(params, call_jaxpr=convert_constvars_jaxpr(jaxpr))
-    if primitive.map_primitive:
-      in_axes = params['in_axes']
-      # NOTE: const_tracers are added as map outputs, and we always map them
-      #       along axis 0 (see `new_out_axes_thunk` above).
-      new_in_axes = ((0,) * len(const_tracers) +
-                     (None,) * len(env_tracers) +
-                     tuple(axis for axis, t in zip(in_axes, tracers)
-                           if not t.pval.is_known()))
-      new_out_axes = tuple(axis for axis, pval in zip(out_axes, out_pvals)
-                           if not pval.is_known())
-      new_params = dict(new_params, in_axes=new_in_axes, out_axes=new_out_axes)
-      del new_params['out_axes_thunk']
-    update_params = call_param_updaters.get(primitive)
-    if update_params:
-      new_params = update_params(new_params, [not t.pval.is_known() for t in tracers])
+      # Set up new params
+      new_params = dict(params, call_jaxpr=convert_constvars_jaxpr(jaxpr))
+      if primitive.map_primitive:
+        in_axes = params['in_axes']
+        # NOTE: const_tracers are added as map outputs, and we always map them
+        #       along axis 0 (see `new_out_axes_thunk` above).
+        new_in_axes = ((0,) * len(const_tracers) +
+                       (None,) * len(env_tracers) +
+                       tuple(axis for axis, t in zip(in_axes, tracers)
+                             if not t.pval.is_known()))
+        new_out_axes = tuple(axis for axis, pval in zip(out_axes, out_pvals)
+                             if not pval.is_known())
+        new_params = dict(new_params, in_axes=new_in_axes, out_axes=new_out_axes)
+        del new_params['out_axes_thunk']
+      update_params = call_param_updaters.get(primitive)
+      if update_params:
+        new_params = update_params(new_params, [not t.pval.is_known() for t in tracers])
 
     eqn = new_eqn_recipe(in_tracers, unknown_tracers_out, primitive, new_params,
                          source_info_util.current())
@@ -644,25 +648,25 @@ def tracers_to_jaxpr(
   const_vars, const_vals = unzip2(consts.items())
   # The env_vars are pre-pended to the invars
   jaxpr = Jaxpr(const_vars, [*env_vars, *invars], map(getvar, out_tracers), eqns)
-  core.skip_checks or core.check_jaxpr(jaxpr)
+  config.jax_enable_checks and core.check_jaxpr(jaxpr)
   return jaxpr, const_vals, env_vals
 
 @cache()
 def convert_constvars_jaxpr(jaxpr: Jaxpr):
   """Moves the constvars to the start of invars."""
-  core.skip_checks or core.check_jaxpr(jaxpr)
+  config.jax_enable_checks and core.check_jaxpr(jaxpr)
   lifted_jaxpr = Jaxpr(constvars=(),
                        invars=jaxpr.constvars + jaxpr.invars,
                        outvars=jaxpr.outvars, eqns=jaxpr.eqns)
-  core.skip_checks or core.check_jaxpr(lifted_jaxpr)
+  config.jax_enable_checks and core.check_jaxpr(lifted_jaxpr)
   return lifted_jaxpr
 
 def convert_envvars_to_constvars(jaxpr: Jaxpr, num_env_vars: int):
-  core.skip_checks or core.check_jaxpr(jaxpr)
+  config.jax_enable_checks and core.check_jaxpr(jaxpr)
   env_vars, invars = split_list(jaxpr.invars, [num_env_vars])
   converted_jaxpr = Jaxpr(constvars=jaxpr.constvars + env_vars,
                           invars=invars, outvars=jaxpr.outvars, eqns=jaxpr.eqns)
-  core.skip_checks or core.check_jaxpr(converted_jaxpr)
+  config.jax_enable_checks and core.check_jaxpr(converted_jaxpr)
   return converted_jaxpr
 
 
@@ -982,11 +986,8 @@ class JaxprStackFrame:
 def _inline_literals(jaxpr, constvals):
   consts = dict(zip(jaxpr.constvars, constvals))
   newvar = core.gensym()
-  class var(dict):
-    def __missing__(self, v):
-      new_v = self[v] = newvar(v.aval)
-      return new_v
-  var = var()
+  newvars = {}
+  var = lambda v: newvars.get(v) or newvars.setdefault(v, newvar(v.aval))
 
   def lit(var: core.Var) -> Optional[Any]:
     val = consts.get(var)
@@ -996,14 +997,21 @@ def _inline_literals(jaxpr, constvals):
       return None
 
   used = {v for eqn in jaxpr.eqns for v in eqn.invars} | set(jaxpr.outvars)
-  new_constvars = [var[v] for v in jaxpr.constvars if not lit(v)]
+  new_constvars = [var(v) for v in jaxpr.constvars if not lit(v)]
   new_constvals = [c for v, c in zip(jaxpr.constvars, constvals) if not lit(v)]
-  new_invars = [var[v] for v in jaxpr.invars]
-  new_eqns = [new_jaxpr_eqn([lit(v) or var[v] for v in eqn.invars],
-                            [var[v] if v in used else dropvar for v in eqn.outvars],
-                            eqn.primitive, eqn.params, eqn.source_info)
-              for eqn in jaxpr.eqns]
-  new_outvars = [lit(v) or var[v] for v in jaxpr.outvars]
+  new_invars = [var(v) for v in jaxpr.invars]
+  new_eqns = []
+  for eqn in jaxpr.eqns:
+    invars = [lit(v) or var(v) for v in eqn.invars]
+    if (eqn.primitive is core.convert_element_type_p and type(invars[0]) is Literal):
+      # constant-fold dtype conversion of literals to be inlined
+      consts[eqn.outvars[0]] = np.array(invars[0].val, eqn.params['new_dtype'])
+    else:
+      # might do DCE here, but we won't until we're more careful about effects
+      outvars = [var(v) if v in used else dropvar for v in eqn.outvars]
+      new_eqns.append(new_jaxpr_eqn(invars, outvars, eqn.primitive, eqn.params,
+                                    eqn.source_info))
+  new_outvars = [lit(v) or var(v) for v in jaxpr.outvars]
   new_jaxpr = Jaxpr(new_constvars, new_invars, new_outvars, new_eqns)
   return new_jaxpr, new_constvals
 
@@ -1100,25 +1108,25 @@ class DynamicJaxprTrace(core.Trace):
     with core.extend_axis_env(axis_name, axis_size, None):  # type: ignore
       jaxpr, reduced_out_avals, consts = trace_to_subjaxpr_dynamic(
           f, self.main, reduced_in_avals)
-    out_axes = params['out_axes_thunk']()
-    out_avals = [core.unmapped_aval(params['axis_size'], out_axis, a)
-                 if out_axis is not None else a
-                 for a, out_axis in zip(reduced_out_avals, out_axes)]
-    source_info = source_info_util.current()
-    out_tracers = [DynamicJaxprTracer(self, a, source_info) for a in out_avals]
-    invars = map(self.getvar, tracers)
-    constvars = map(self.getvar, map(self.instantiate_const, consts))
-    outvars = map(self.makevar, out_tracers)
-    new_in_axes = (None,) * len(consts) + params['in_axes']
-    new_params = dict(params, in_axes=new_in_axes, out_axes=out_axes,
-                      call_jaxpr=convert_constvars_jaxpr(jaxpr))
-    del new_params['out_axes_thunk']
-    update_params = call_param_updaters.get(map_primitive)
-    if update_params:
-      new_params = update_params(new_params, [True] * len(tracers))
-    eqn = new_jaxpr_eqn([*constvars, *invars], outvars, map_primitive,
-                        new_params, source_info)
-    self.frame.eqns.append(eqn)
+      out_axes = params['out_axes_thunk']()
+      out_avals = [core.unmapped_aval(params['axis_size'], out_axis, a)
+                  if out_axis is not None else a
+                  for a, out_axis in zip(reduced_out_avals, out_axes)]
+      source_info = source_info_util.current()
+      out_tracers = [DynamicJaxprTracer(self, a, source_info) for a in out_avals]
+      invars = map(self.getvar, tracers)
+      constvars = map(self.getvar, map(self.instantiate_const, consts))
+      outvars = map(self.makevar, out_tracers)
+      new_in_axes = (None,) * len(consts) + params['in_axes']
+      new_params = dict(params, in_axes=new_in_axes, out_axes=out_axes,
+                        call_jaxpr=convert_constvars_jaxpr(jaxpr))
+      del new_params['out_axes_thunk']
+      update_params = call_param_updaters.get(map_primitive)
+      if update_params:
+        new_params = update_params(new_params, [True] * len(tracers))
+      eqn = new_jaxpr_eqn([*constvars, *invars], outvars, map_primitive,
+                          new_params, source_info)
+      self.frame.eqns.append(eqn)
     return out_tracers
 
   def post_process_map(self, map_primitive, out_tracers, params):
