@@ -23,11 +23,10 @@ from warnings import warn
 from absl import logging
 import numpy as np
 
-from ..config import flags, bool_env, config
+from ..config import config
 from .. import core
 from .. import ad_util
 from .. import dtypes
-from .. import lazy
 from .. import linear_util as lu
 from jax._src import source_info_util
 from ..abstract_arrays import (make_shaped_array, array_types)
@@ -36,6 +35,7 @@ from ..core import (ConcreteArray, ShapedArray, AbstractToken,
 from jax._src.pprint_util import pp
 from .._src.util import (partial, partialmethod, cache, prod, unzip2,
                     extend_name_stack, wrap_name, safe_zip, safe_map)
+from .. import lib
 from ..lib import xla_bridge as xb
 from ..lib import xla_client as xc
 from . import partial_eval as pe
@@ -58,17 +58,6 @@ XlaShape = Any # xla_client.Shape
 XlaComputationBuilder = Any  # xla_bridge._JaxComputationBuilder
 XlaExecutable = Any # xla_extension.LocalExecutable
 
-FLAGS = flags.FLAGS
-flags.DEFINE_bool('jax_debug_nans',
-                  bool_env('JAX_DEBUG_NANS', False),
-                  'Add nan checks to every operation.')
-flags.DEFINE_bool('jax_debug_infs',
-                  bool_env('JAX_DEBUG_INFS', False),
-                  'Add inf checks to every operation.')
-flags.DEFINE_bool('jax_log_compiles',
-                  bool_env('JAX_LOG_COMPILES', False),
-                  'Print a message each time a `jit` computation is compiled.')
-
 # This flag is set on exit; no logging should be attempted
 _on_exit = False
 
@@ -114,7 +103,7 @@ def aval_to_result_handler(device: Optional[Device], aval: core.AbstractValue) -
 def array_result_handler(device: Optional[Device], aval: core.ShapedArray):
   if aval.dtype is dtypes.float0:
     return lambda _: np.zeros(aval.shape, dtypes.float0)
-  return partial(make_device_array, raise_to_shaped(aval), device, None)
+  return partial(make_device_array, raise_to_shaped(aval), device)
 
 
 xla_result_handlers: Dict[Type[core.AbstractValue], Callable[..., Callable]] = {
@@ -244,7 +233,7 @@ def apply_primitive(prim, *args, **params):
 
 def _partition_outputs(avals, outs):
   nouts = [aval._num_buffers for aval in avals]
-  if not core.skip_checks:
+  if config.jax_enable_checks:
     assert sum(nouts) == len(outs), f"Internal error: sum(nouts)={sum(nouts)} should equal len(outs)={len(outs)}."
   outs = iter(outs)
   return [[next(outs) for _ in range(nout)] for nout in nouts]
@@ -358,7 +347,7 @@ def _execute_compiled_primitive(prim, compiled, result_handler, *args):
   device, = compiled.local_devices()
   input_bufs = list(it.chain.from_iterable(device_put(x, device) for x in args if x is not token))
   out_bufs = compiled.execute(input_bufs)
-  check_special(prim, out_bufs)
+  check_special(prim.name, out_bufs)
   return result_handler(*out_bufs)
 
 def _execute_replicated_primitive(prim, compiled, result_handler, *args):
@@ -371,18 +360,20 @@ def _execute_replicated_primitive(prim, compiled, result_handler, *args):
   ]
   return result_handler(*out_bufs)
 
+def needs_check_special():
+  return config.jax_debug_infs or config.jax_debug_nans
 
-def check_special(prim, bufs):
-  if FLAGS.jax_debug_infs or FLAGS.jax_debug_nans:
+def check_special(name, bufs):
+  if needs_check_special():
     for buf in bufs:
-      _check_special(prim.name, buf.xla_shape(), buf)
+      _check_special(name, buf.xla_shape(), buf)
 
 def _check_special(name, xla_shape, buf):
   assert not xla_shape.is_tuple()
   if dtypes.issubdtype(xla_shape.element_type(), np.inexact):
-    if FLAGS.jax_debug_nans and np.any(np.isnan(buf.to_py())):
+    if config.jax_debug_nans and np.any(np.isnan(buf.to_py())):
       raise FloatingPointError(f"invalid value (nan) encountered in {name}")
-    if FLAGS.jax_debug_infs and np.any(np.isinf(buf.to_py())):
+    if config.jax_debug_infs and np.any(np.isinf(buf.to_py())):
       raise FloatingPointError(f"invalid value (inf) encountered in {name}")
 
 ### compiling jaxprs
@@ -588,13 +579,13 @@ def _xla_call_impl(fun: lu.WrappedFun, *args, device, backend, name, donated_inv
   try:
     return compiled_fun(*args)
   except FloatingPointError:
-    assert FLAGS.jax_debug_nans or FLAGS.jax_debug_infs  # compiled_fun can only raise in this case
+    assert config.jax_debug_nans or config.jax_debug_infs  # compiled_fun can only raise in this case
     print("Invalid value encountered in the output of a jit function. "
           "Calling the de-optimized version.")
     # We want to run the wrapped function again (after _xla_callable already ran
     # it), but linear_util.WrappedFun instances are meant to be run only once.
     # In addition to re-executing the Python code, which is usually undesirable
-    # but which FLAGS.jax_debug_nans is meant to opt into, we'll be re-executing
+    # but which config.jax_debug_nans is meant to opt into, we'll be re-executing
     # any linear_util.py-style side effects, i.e. re-populating Stores created
     # by any transformation_with_aux's applied to fun. Since this is
     # intentional here, to avoid "Store occupied" errors we reset the stores to
@@ -632,18 +623,19 @@ def flatten_shape(s: XlaShape) -> Sequence[Tuple[Sequence[int], XlaShape]]:
     An iterable of pairs of indices and shapes for each array within the shape
     tree.
   """
-  def _flatten_shape(s, index):
-    if s.is_array():
-      yield index, s
-    else:
-      assert s.is_tuple()
-      for i, sub in enumerate(s.tuple_shapes()):
-        subindex = index + (i,)
-        if sub.is_tuple():
-          yield from _flatten_shape(sub, subindex)
-        else:
-          yield subindex, sub
-  return tuple(_flatten_shape(s, index=()))
+  results: List[Tuple[Tuple[int, ...], XlaShape]] = []
+  _flatten_shape(s, (), results)
+  return results
+
+def _flatten_shape(s: XlaShape, index: Tuple[int, ...],
+                   results: List[Tuple[Tuple[int, ...], XlaShape]]) -> None:
+  if s.is_array() or s.is_token():
+    results.append((index, s))
+  else:
+    assert s.is_tuple()
+    for i, sub in enumerate(s.tuple_shapes()):
+      _flatten_shape(sub, index + (i,), results)
+
 
 def _xla_consts(c, consts):
   unique_consts = {id(const): const for const in consts}
@@ -680,12 +672,12 @@ def _xla_callable(fun: lu.WrappedFun, device, backend, name, donated_invars, *ar
 
   # Computations that only produce constants and/or only rearrange their inputs,
   # which are often produced from partial evaluation, don't need compilation,
-  # and don't need to force their (potentially lazy) arguments.
+  # and don't need to evaluate their arguments.
   if not jaxpr.eqns:
     return partial(_execute_trivial, jaxpr, device, consts, out_avals, result_handlers)
 
   if not _on_exit:
-    log_priority = logging.WARNING if FLAGS.jax_log_compiles else logging.DEBUG
+    log_priority = logging.WARNING if config.jax_log_compiles else logging.DEBUG
     logging.log(log_priority, "Compiling %s (%s) for args %s.",
                 fun.__name__, id(fun), abstract_args)
 
@@ -745,7 +737,7 @@ def set_up_aliases(c, xla_args, out_tuple, donated_args, tuple_args):
   for arg_index, arg in enumerate(xla_args):
     if donated_args[arg_index]:
       for param_index, element in flatten_shape(c.GetShape(arg)):
-        key = (element.dimensions(), element.numpy_dtype())
+        key = (element.dimensions(), element.xla_element_type())
         if tuple_args:
           param_number = 0
           param_index = (arg_index,) + tuple(param_index)
@@ -757,7 +749,7 @@ def set_up_aliases(c, xla_args, out_tuple, donated_args, tuple_args):
   # Consume donations for outputs.
   out_donated_args = list(donated_args)
   for output_index, element in flatten_shape(c.GetShape(out_tuple)):
-    key = (element.dimensions(), element.numpy_dtype())
+    key = (element.dimensions(), element.xla_element_type())
     if donations.get(key, ()):
       param_number, param_index, arg_index = donations[key].popleft()
       out_donated_args[arg_index] = False
@@ -846,7 +838,7 @@ def _execute_compiled(compiled: XlaExecutable, avals, handlers, *args):
   device, = compiled.local_devices()
   input_bufs = list(it.chain.from_iterable(device_put(x, device) for x in args if x is not token))
   out_bufs = compiled.execute(input_bufs)
-  check_special(xla_call_p, out_bufs)
+  check_special(xla_call_p.name, out_bufs)
   return [handler(*bs) for handler, bs in zip(handlers, _partition_outputs(avals, out_bufs))]
 
 def _execute_replicated(compiled: XlaExecutable, avals, handlers, *args):
@@ -857,7 +849,7 @@ def _execute_replicated(compiled: XlaExecutable, avals, handlers, *args):
       buf[0] for buf in compiled.execute_sharded_on_local_devices(
           list(zip(*input_bufs)))
   ]
-  check_special(xla_call_p, out_bufs)
+  check_special(xla_call_p.name, out_bufs)
   return [handler(*bs) for handler, bs in zip(handlers, _partition_outputs(avals, out_bufs))]
 
 def _execute_trivial(jaxpr, device: Optional[Device], consts, avals, handlers, *args):
@@ -1027,13 +1019,9 @@ DeviceArray = xc.DeviceArrayBase
 
 _CppDeviceArray: DeviceArrayProtocol = xc.Buffer
 
-_EXPERIMENTAL_CPP_DEVICE_ARRAY = False
-
-
 def make_device_array(
     aval: core.ShapedArray,
     device: Optional[Device],
-    lazy_expr: Optional[lazy.LazyExpr],
     device_buffer: PyLocalBuffer,
 ) -> Union[PyLocalBuffer, "_DeviceArray"]:
   """Returns a DeviceArray implementation based on arguments.
@@ -1041,13 +1029,17 @@ def make_device_array(
   This is to be used only within JAX. It will return either a PythonDeviceArray
   or a C++ equivalent implementation.
   """
-  if _EXPERIMENTAL_CPP_DEVICE_ARRAY and lazy.is_trivial(lazy_expr):
-    assert isinstance(device_buffer, _CppDeviceArray)
-    device_buffer._device = device    # pylint: disable=protected-access
+  if (isinstance(device_buffer, _CppDeviceArray)):
+
+    if device_buffer.aval == aval and device_buffer._device == device:
+      return device_buffer
+    device_buffer = device_buffer.clone()
+    device_buffer._device = device
     device_buffer.aval = aval
+    device_buffer.weak_type = aval.weak_type
     return device_buffer
 
-  return _DeviceArray(aval, device, lazy_expr, device_buffer)
+  return _DeviceArray(aval, device, device_buffer)
 
 
 def type_is_device_array(x):
@@ -1064,7 +1056,7 @@ class _DeviceArray(DeviceArray):  # type: ignore
   # We don't subclass ndarray because that would open up a host of issues,
   # but lax_numpy.py overrides isinstance behavior and attaches ndarray methods.
   __slots__ = [
-      "aval", "device_buffer", "_npy_value", "_device", "_lazy_expr"
+      "aval", "device_buffer", "_npy_value", "_device"
   ]
   __array_priority__ = 100
 
@@ -1073,7 +1065,6 @@ class _DeviceArray(DeviceArray):  # type: ignore
   _HAS_DYNAMIC_ATTRIBUTES = True
 
   def __init__(self, aval: core.ShapedArray, device: Optional[Device],
-               lazy_expr: Optional[lazy.LazyExpr],
                device_buffer: PyLocalBuffer):
     """Initializer.
 
@@ -1081,18 +1072,15 @@ class _DeviceArray(DeviceArray):  # type: ignore
       aval: The abstract value associated to this array (shape+dtype+weak_type).
       device:  The optional sticky device. See
         https://jax.readthedocs.io/en/latest/faq.html#controlling-data-and-computation-placement-on-devices
-      lazy_expr: An optional `LayExpr`. `None` is equivalent to a trivial
-        `LazyExpr`.
       device_buffer: The underlying buffer owning the on-device data.
     """
     DeviceArray.__init__(self)
     self.aval = aval
     self.device_buffer = device_buffer
     self._device = device
-    self._lazy_expr = lazy_expr
 
     self._npy_value = None
-    if not core.skip_checks:
+    if config.jax_enable_checks:
       assert type(aval) is ShapedArray
       npy_value = self._value
       assert npy_value.dtype == aval.dtype and npy_value.shape == aval.shape
@@ -1119,7 +1107,7 @@ class _DeviceArray(DeviceArray):  # type: ignore
   def _value(self):
     self._check_if_deleted()
     if self._npy_value is None:
-      self._npy_value = _force(self).device_buffer.to_py()
+      self._npy_value = self.device_buffer.to_py()
       self._npy_value.flags.writeable = False
     return self._npy_value
 
@@ -1162,7 +1150,7 @@ class _DeviceArray(DeviceArray):  # type: ignore
 
   @property
   def __cuda_array_interface__(self):
-    return _force(self).device_buffer.__cuda_array_interface__
+    return self.device_buffer.__cuda_array_interface__
 
 
 # Adding methods dynamically to both _DeviceArray and _CppDeviceArray
@@ -1278,9 +1266,6 @@ for device_array in [DeviceArray]:
 class DeletedBuffer(object): pass
 deleted_buffer = DeletedBuffer()
 
-# TODO(phawkins): remove DeviceConstant after updating users.
-class DeviceConstant(object): pass
-
 for device_array in [_CppDeviceArray, _DeviceArray]:
   core.literalable_types.add(device_array)
   core.pytype_aval_mappings[device_array] = ConcreteArray
@@ -1288,14 +1273,13 @@ for device_array in [_CppDeviceArray, _DeviceArray]:
   canonicalize_dtype_handlers[device_array] = identity
 
 def _device_array_constant_handler(c, val, canonicalize_types=True):
-  base_val = xb.constant(c, val.device_buffer.to_py())
-  return lazy.stage_lexpr(c, val._lazy_expr, base_val)
+  return xb.constant(c, val.device_buffer.to_py())
 xb.register_constant_handler(_DeviceArray, _device_array_constant_handler)
 xb.register_constant_handler(_CppDeviceArray, _device_array_constant_handler)
 
 def _device_put_device_array(x: Union[DeviceArrayProtocol, _DeviceArray], device: Optional[Device]):
   x = _copy_device_array_to_device(x, device)
-  return (_force(x).device_buffer,)
+  return (x.device_buffer,)
 device_put_handlers[_CppDeviceArray] = _device_put_device_array
 device_put_handlers[_DeviceArray] = _device_put_device_array
 
@@ -1318,44 +1302,7 @@ def _copy_device_array_to_device(x: Union[DeviceArrayProtocol, _DeviceArray], de
     # buffers from different XLA backends are passed through the host.
     backend = xb.get_device_backend(device)
     moved_buf = backend.buffer_from_pyval(x.device_buffer.to_py(), device)
-  return _DeviceArray(x.aval, device, x._lazy_expr, moved_buf)
-
-def _force(x: DeviceArrayProtocol) -> DeviceArrayProtocol:
-  if lazy.is_trivial(x._lazy_expr):
-    return x
-  else:
-    # force x on the device where it lives, but preserve stickiness on result
-    if x._device:
-      device = x._device
-    else:
-      device = x.device_buffer.device()
-    force_fun = _lazy_force_computation(x.aval, device, x._lazy_expr)
-    result = force_fun(x)
-    return make_device_array(x.aval, x._device, None, result)
-
-@cache()
-def _lazy_force_computation(aval: core.ShapedArray,
-                            device: Device, lexpr: lazy.LazyExpr
-                            ) -> Callable[[_DeviceArray], PyLocalBuffer]:
-  c = xb.make_computation_builder("lazy_force")
-  idxs = [(src, dst) for dst, src in enumerate(lexpr.dims) if src is not None]
-  param_shape = [None] * len(idxs)
-  for src, dst in idxs:
-    param_shape[src] = aval.shape[dst]
-  param = xb.parameter(c, 0, xc.Shape.array_shape(aval.dtype, param_shape))
-  xla_out = lazy.stage_lexpr(c, lexpr, param)
-  built_c = c.build(xla_out)
-
-  device = _device_from_arg_devices([device])
-  options = xb.get_compile_options(
-      num_replicas=1,
-      num_partitions=1,
-      device_assignment=device and (device.id,))
-  compiled = backend_compile(xb.get_device_backend(device), built_c, options)
-
-  def force_fun(x: _DeviceArray) -> PyLocalBuffer:
-    return compiled.execute([x.device_buffer])[0]
-  return force_fun
+  return make_device_array(x.aval, device, moved_buf)
 
 
 def _device_put_impl(x, device: Optional[Device] = None):
