@@ -20,24 +20,30 @@ __all__ = [
   'pocketfft', 'pytree', 'tpu_client', 'version', 'xla_client'
 ]
 
-import jaxlib
+try:
+  import jaxlib
+except ModuleNotFoundError as err:
+  raise ModuleNotFoundError(
+    'jax requires jaxlib to be installed. See '
+    'https://github.com/google/jax#installation for installation instructions.'
+    ) from err
 
-# Must be kept in sync with the jaxlib version in build/test-requirements.txt
-_minimum_jaxlib_version = (0, 1, 60)
+from jax.version import _minimum_jaxlib_version as _minimum_jaxlib_version_str
 try:
   from jaxlib import version as jaxlib_version
 except Exception as err:
   # jaxlib is too old to have version number.
-  msg = 'This version of jax requires jaxlib version >= {}.'
-  raise ImportError(msg.format('.'.join(map(str, _minimum_jaxlib_version)))
-                    ) from err
+  msg = f'This version of jax requires jaxlib version >= {_minimum_jaxlib_version_str}.'
+  raise ImportError(msg) from err
 
 version = tuple(int(x) for x in jaxlib_version.__version__.split('.'))
+_minimum_jaxlib_version = tuple(int(x) for x in _minimum_jaxlib_version_str.split('.'))
 
 # Check the jaxlib version before importing anything else from jaxlib.
 def _check_jaxlib_version():
   if version < _minimum_jaxlib_version:
-    msg = 'jaxlib is version {}, but this version of jax requires version {}.'
+    msg = (f'jaxlib is version {jaxlib_version.__version__}, '
+           f'but this version of jax requires version {_minimum_jaxlib_version_str}.')
 
     if version == (0, 1, 23):
       msg += ('\n\nA common cause of this error is that you installed jaxlib '
@@ -45,8 +51,7 @@ def _check_jaxlib_version():
               'manylinux2010 wheels. Try running:\n\n'
               'pip install --upgrade pip\n'
               'pip install --upgrade jax jaxlib\n')
-    raise ValueError(msg.format('.'.join(map(str, version)),
-                                '.'.join(map(str, _minimum_jaxlib_version))))
+    raise ValueError(msg)
 
 _check_jaxlib_version()
 
@@ -84,3 +89,48 @@ try:
   from jaxlib import tpu_client  # pytype: disable=import-error
 except:
   tpu_client = None
+
+
+# Backwards compatibility shims for the C++ jax_jit.global_state and
+# jax_jit.thread_local_state which were introduced in XLA extension version 9
+# (jaxlib 0.1.64).
+if _xla_extension_version <= 8:
+  class GlobalJitState:
+    @property
+    def disable_jit(self):
+      return jax_jit.get_disable_jit_cpp_flag()
+
+    @disable_jit.setter
+    def disable_jit(self, x):
+      jax_jit.set_disable_jit_cpp_flag(x)
+
+    @property
+    def enable_x64(self):
+      return jax_jit.get_enable_x64_cpp_flag()
+
+    @enable_x64.setter
+    def enable_x64(self, x):
+      jax_jit.set_enable_x64_cpp_flag(x)
+
+  global_state = GlobalJitState()
+  jax_jit.global_state = lambda: global_state
+
+
+  class ThreadLocalJitState:
+    @property
+    def disable_jit(self):
+      return jax_jit.get_disable_jit_thread_local()
+
+    @disable_jit.setter
+    def disable_jit(self, x):
+      jax_jit.set_disable_jit_thread_local(x)
+
+    @property
+    def enable_x64(self):
+      return jax_jit.get_enable_x64_thread_local()
+
+    @enable_x64.setter
+    def enable_x64(self, x):
+      jax_jit.set_enable_x64_thread_local(x)
+  thread_local_state = ThreadLocalJitState()
+  jax_jit.thread_local_state = lambda: thread_local_state
